/* eslint-disable */
import {
    Http,
    Metadata,
} from '@zeedhi/core';
import {
    DialogService, AlertService, FileInput, Form, LoadingService,
    Row,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export interface ElementChange {
    ID: number;
    PARENT_ID?: number;
    ATIVO: string;
    PARCIAL?: boolean;
}

export class Utilities {
    public static readonly MSG_CAMPOS_NAO_PREENCHIDOS = 'Existem campos obrigatórios não preenchidos!';

    public static alertMessage(message: string, type: string) {
        AlertService.show({
            name: 'basic-alert',
            text: message,
            top: true,
            left: false,
            timeout: 3000,
            color: type,
        });
    }

    public static showErrorMessage(error: any) {
        console.error(error);
        let msg = '';
        if (error.response && error.response.data && error.response.data.error) {
            msg = error.response.data.error;
        } else {
            msg = error;
        }
        if (!msg) {
            msg = 'Ocorreu um erro inesperado. Por favor, entre em contato com o suporte.';
        }

        if (msg.includes("ORA-02292")) {
            msg = "Exclusão não permitida! O registro possui registros filhos.";
        }

        this.showDialogMessage('error', 'Erro', msg);
    }

    /**
     * Mostra uma caixa de mensagem dialog.
     * 
     * @param type tipo da mensagem (success, info, warning, error)
     * @param title titulo da caixa de mensagem
     * @param text conteudo da caixa de mensagem
     */
    public static showDialogMessage(type: string, title: string, text: string) {
        DialogService.show({
            name: type + '_dialog',
            type: type,
            title: title,
            text: text,
        });
    }

    public static async salvarDados(rota: any, parametros: any) {
        await Http.post(rota, parametros).then(() => {
            this.showSuccessMessage();
        }).catch((error: any) => {
            this.showErrorMessage(error);
        });
    }

    public static showSuccessMessage() {
        AlertService.show({
            name: 'basic-alert',
            text: 'Processo realizado com sucesso!',
            top: true,
            left: false,
            color: 'success',
            timeout: 2000,
            buttonsSlot: [
                {
                    name: 'undo-button',
                    component: 'ZdButton',
                    icon: true,
                    right: true,
                    iconName: 'close',
                    color: 'white',
                    events: {
                        click: () => { AlertService.hide(); },
                    },
                },
            ],
        });
    }

    public static async buildArrayToUpload(nomeComponente: string) {
        const file = Metadata.getInstance<FileInput>(nomeComponente).value;
        if (file) {
            const base64file = await this.toBase64(file);
            return [{
                b64File: base64file,
                lastModified: file.lastModified,
                name: file.name,
                size: file.size,
                type: file.type,
                webkitRelativePath: file.webkitRelativePath,
            }];
        }
        return [];
    }

    /* public static async buildArrayToUploadPromise(nomeComponente: string) {
        const file = Metadata.getInstance<FileInput>(nomeComponente).value;
        return new Promise((resolve, reject) => {
            if (file) {
                this.toBase64(file).then((base64file) => {
                    resolve([{
                        b64File: base64file,
                        lastModified: file.lastModified,
                        name: file.name,
                        size: file.size,
                        type: file.type,
                        webkitRelativePath: file.webkitRelativePath,
                    }]);
                }).catch((error) => {
                    reject(error);
                });
            }
            else {
                resolve([]);
            }
        });
    }; */

    public static async toBase64(file: any) {
        return new Promise((resolve, reject) => {
            const reader = new FileReader();
            reader.readAsDataURL(file);
            reader.onload = () => resolve(reader.result);
            reader.onerror = (error) => reject(error);
        });
    }

    public static showConfirmDeleteMessage(event: any, functionConfirm: string) {
        event.preventDefault();
        functionConfirm = '{{' + functionConfirm + '}}';
        DialogService.show({
            name: 'buttons-dialog',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                {
                    name: 'button-no',
                    component: 'ZdButton',
                    label: 'Não',
                    events: {
                        click: () => DialogService.hide(),
                    },
                    outline: true,
                },
                {
                    name: 'button-yes',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: functionConfirm
                    },
                },
            ],
        });
    }

    public static showDialogConfirm(title: string, event: any, functionConfirm: string, message: string, disableButton: boolean) {
        event.preventDefault();
        functionConfirm = '{{' + functionConfirm + '}}';
        DialogService.show({
            name: 'buttons-dialog',
            type: 'warning',
            title: title,
            text: message,
            buttons: [
                {
                    name: 'button-no',
                    component: 'ZdButton',
                    label: 'Cancelar',
                    events: {
                        click: () => DialogService.hide(),
                    },
                    outline: true,
                },
                {
                    name: 'button-yes',
                    component: 'ZdButton',
                    label: 'Confirmar',
                    disabled: disableButton,
                    events: {
                        click: functionConfirm
                    },
                },
            ],
        });
    }

    public static getOperadorLogado() {
        return Http.post('/getOperadorLogado', {}).then((response) => {
            return new Promise((resolve, reject) => {
                resolve(response.data.dataset.retorno.cdoperador);
            });
        });
    }

    public static post(route: string, parameters = {}) {
        LoadingService.show();
        return new Promise((resolve, reject) => {
            Http.post(route, parameters).then((response) => {
                this.showSuccessMessage();
                resolve(response);
            }).catch((error: any) => {
                this.showErrorMessage(error);
                reject(error);
            }).finally(() => {
                LoadingService.hide();
            });
        });
    }

    public static get(route: string) {
        LoadingService.show();
        return new Promise((resolve, reject) => {
            Http.get(route).then((response) => {
                this.showSuccessMessage();
                resolve(response);
            }).catch((error: any) => {
                this.showErrorMessage(error);
                reject(error);
            }).finally(() => {
                LoadingService.hide();
            });
        });
    }

    public static validateForm(form: Form) {
        if (form.validate()) {
            return true;
        }
        this.showAlertCamposObrigatorios();
        return false;
    }

    public static validateGrid(grid: TekGrid) {
        if (grid.isGridValid(true)) {
            return true;
        }
        this.showAlertCamposObrigatorios();
        return false;
    }

    public static showAlertCamposObrigatorios() {
        this.alertMessage(this.MSG_CAMPOS_NAO_PREENCHIDOS, 'error');
    }

    public static async timeout(time: number) {
        return new Promise((resolve) => setTimeout(resolve, time));
    }

    public static async doAsync(callBack: Function, time: number) {
        return new Promise<void>((resolve) => {
            this.timeout(time).then(() => {
                callBack();
                resolve();
            });
        });
    }

    /* Retorna a quantidade de dias entre um intervalo de datas. Caso a data final seja maior que a data inicial, o
     * valor retornado sera negativo, contendo a quantidade de dias entre as duas datas.
    */

    public static getQtdeDiasEntre(dataInicial: string | Date, dataFinal: string | Date): number {
        // Converte as datas no formato "DD/MM/AAAA" para "AAAA-MM-DD"
        const dataInicialFormatada = typeof dataInicial === 'string' ? dataInicial.split('/').reverse().join('-') : dataInicial;
        const dataFinalFormatada = typeof dataFinal === 'string' ? dataFinal.split('/').reverse().join('-') : dataFinal;

        const dataInicialObj = new Date(dataInicialFormatada);
        const dataFinalObj = new Date(dataFinalFormatada);

        const diffTime = dataFinalObj.getTime() - dataInicialObj.getTime();
        const diffDays = Math.floor(diffTime / (1000 * 60 * 60 * 24));

        return diffDays;
    }

    public static getNameDynamic(name: string) {
        const date = new Date();
        return name + date.getTime();
    }

    public static formatDateHoursString(dataString: string): string {
        if (dataString !== null) {
            const partes = dataString.split(' ');
            const dataPartes = partes[0].split('/');
            const dataFormatada = `${dataPartes[0]}/${dataPartes[1]}/${dataPartes[2]}`;
            return dataFormatada;
        }
        return '';
    }

    public static getDates(): string[] {
        const currentDate = new Date();

        // Calcula a data de 6 meses atrás
        const sixMonthsAgo = new Date(currentDate);
        sixMonthsAgo.setMonth(currentDate.getMonth() - 6);

        // Formata as datas no formato DD/MM/YYYY
        const formattedSixMonthsAgo = this.formatDate(sixMonthsAgo);
        const formattedCurrentDate = this.formatDate(currentDate);

        // Retorna o array com as datas formatadas
        return [formattedSixMonthsAgo, formattedCurrentDate];
    }

    public static formatDate(date: Date): string {
        const day = ('0' + date.getDate()).slice(-2);
        const month = ('0' + (date.getMonth() + 1)).slice(-2);
        const year = date.getFullYear();
        return `${day}/${month}/${year}`;
    }

    public static firstDayOfCurrentMonth(): Date {
        const now = new Date();
        return new Date(now.getFullYear(), now.getMonth(), 1);
    }

    public static loadViewButtonCss = function (elements) {
        elements.forEach(function (element) {
            element.PARCIAL = false;
            const children: ElementChange[] = [];
            const active: ElementChange[] = [];
            const inactive: ElementChange[] = [];

            elements.forEach((child) => {
                if (element.ID === child.PARENT_ID) {
                    children.push(child);
                }
            });

            if (children.length) {
                children.forEach((child) => {
                    if ((child.ATIVO === 'S' && inactive.length) || child.PARCIAL) {
                        element.PARCIAL = true;
                        if (element.PARENT_ID) {
                            elements.forEach((parent) => {
                                if (element.PARENT_ID === parent.ID) {
                                    parent.PARCIAL = true;
                                }
                            });
                        }
                    } else if (child.ATIVO === 'S') {
                        active.push(child);
                    } else if (child.ATIVO === 'N' && active.length) {
                        element.PARCIAL = true;
                        if (element.PARENT_ID) {
                            elements.forEach((parent) => {
                                if (element.PARENT_ID === parent.ID) {
                                    parent.PARCIAL = true;
                                }
                            });
                        }
                    } else {
                        inactive.push(child);
                    }
                });

                if (active.length === children.length) {
                    element.PARCIAL = false;
                }
            }
        });
    };
}